package com.putnam.jboss.issues;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.hornetq.api.core.management.HornetQServerControl;
import org.hornetq.api.core.management.ObjectNameBuilder;


/**
 * Use the HQ Mbeans to create a HQ address. A HQ address is the mechanism by which we define max sizes for a JMS queue or 
 * topic, as well as the mechanism by which we define the policy to be implemented when a message expires, etc.
 * 
 * @see http://appintegration.putnaminv.com/docs/JBossDocs/HornetQ/2.1.0.CR1/user-manual/en/html/queue-attributes.html#queue-attributes.address-settings
 * @author aengineer
 *
 */
public class CreateAddressFails {
    private static final String logPrefix = "CreateAddressFails::";
    
    public static void main(String[] args) throws Exception {
        CreateAddressFails client = new CreateAddressFails();
        client.run(args);
    }
     
    private void run(String[] args) throws Exception {
        String logMsgPrefix = logPrefix + "run()::";
        String message = null;
        JMXConnector jmxc = null;

        // Create JMX Connector to connect to the server's MBeanServer
        jmxc = connectJMX("service:jmx:rmi:///jndi/rmi://richmond2:7144/jmxrmi", null, null);

        // Retrieve the MBeanServerConnection
        MBeanServerConnection mbsc = null;
        try {
            mbsc = jmxc.getMBeanServerConnection();
        } catch (IOException ioe) {
            message = logMsgPrefix + "Caught exception when trying to find heap memory usage";
            System.err.println(message);
            throw ioe;
        }

        // Retrieve the ObjectName of the queue. This is used to identify the server resources to manage
        ObjectName objName = ObjectNameBuilder.DEFAULT.getJMSServerObjectName();
        
        // Create a JMSServerControl proxy to create the queue on the server
        HornetQServerControl hqServerControl = (HornetQServerControl)MBeanServerInvocationHandler.newProxyInstance(mbsc, objName, HornetQServerControl.class, false);

        /*
         * Create a address for this single JMS destination.
         */
        boolean lastValueQueue = false;
        int deliveryAttempts = 5;
        long  maxSizeBytes = (50 * 1024 * 1024);
        int pageSizeBytes = (10*1024*1024);
        long redeliveryDelay = 0;
        long redistributionDelay = 0;
        boolean sendToDLAOnNoRoute = true;
        String addressFullMessagePolicy = "BLOCK";
        hqServerControl.addAddressSettings("jms.queue.PUT_DEV_JMX_QUEUE", "jms.queue.DLQ", "jms.queue.ExpiryQueue",
                                           lastValueQueue, deliveryAttempts, maxSizeBytes,
                                           pageSizeBytes,  redeliveryDelay,  redistributionDelay,
                                           sendToDLAOnNoRoute,  addressFullMessagePolicy);
        
        // We close the JMX connector
        closeJMX(jmxc);
        
    }
    
    /**
     * @param service
     * @param user
     * @param password
     * @return
     * @throws Exception
     */
    public JMXConnector connectJMX(String service, String user, String password) throws Exception {
        String logMsgPrefix = logPrefix + "connect::";
        String message = null;
        JMXConnector jmxc = null;
        JMXServiceURL jmxUrl = null;
        
        System.out.println (logMsgPrefix + "Using JMX service URL [" + service + "]");
        try {
            jmxUrl = new JMXServiceURL(service);
        } catch (MalformedURLException mfe) {
            message = logMsgPrefix + "Bad URL";
            System.err.println(message);
            throw mfe;
        }
        
        try {
            Map<String, String[]> env = new HashMap<String, String[]>();
            
            if (user != null && password != null) {
                String[] credentials = new String[] { user, password };
                env.put(JMXConnector.CREDENTIALS, credentials);
            }
            jmxc = JMXConnectorFactory.connect(jmxUrl, env);
        } catch (IOException ioe) {
            message = logMsgPrefix + "Cannot create JMXConnector to service URL [" + service + "] using username [" + user + "]";
            System.err.println(message);
            throw ioe;
        }
        return (jmxc);
    }    

    /**
     * @throws Exception
     */
    public void closeJMX(JMXConnector jmxc) throws Exception {
        String logMsgPrefix = logPrefix + "close::";
        String message = null;
        
        if (jmxc == null) {
            System.err.println (logMsgPrefix + "Unable to close JMX connector. Null passed in.");
            return;
        }
        
        try {
            System.out.println(logMsgPrefix + "Attempting to close JMX connection ");
            jmxc.close();
            jmxc = null;
            System.out.println(logMsgPrefix + "Successfully closed JMX connection");
        } catch (IOException ioe) {
            message = logMsgPrefix + "Failed to close JMX connection";
            System.err.println(message);
            throw ioe;
        }
    }    
}
